<?php

	/**
	 * Api steuert die Bezahlung per Lastschrift.
	 * 
	 *  SEPA Lastschrift:
	 *  Mit SEPA ( Single Euro Payments Area ), dem einheitlichen Euro-Zahlungsverkehrsraum, werden auch in Deutschland zum 01.02.2014 neue,
	 *  europaweit einheitliche Verfahren für den bargeldlosen Zahlungsverkehr (Überweisungen, Lastschriften) eingeführt.
	 * 
	 *  Das bisheriege Verfahren mit Bankleitzahl und Kontonummer wird durch IBAN und BIC abgelöst.
	 * 
	 *  IBAN (International Bank Account Number) ist die internationale Kontonummer und ersetzt Bankleitzahl und Kontonummer.
	 *  Für grenzüberschreitende Zahlungen ist noch der BIC (Business Identifier Code) als international standardisierter
	 *  Bank-Code (vergleichbar mit der Bankleitzahl in Deutschland) erforderlich.
	 * 
	 *  Für Zahlungen / Lastschriften innerhalb Deutschlands reicht die Verwendung der IBAN (iban only) aus, für alle anderen Transaktionen wird IBAN und BIC benötigt.
	 * 
	 *  Für eine ordentliche SEPA-Zahlung sind noch folgende Punkte wichtig:
	 *  Mandate (mandateRef), Gläubiger-ID (creditorId) und Prenotification (Nachricht über die Kundenbelastung; payDate).
	 * 
	 * 
	 *  Mandate:
	 *  Parametername "mandateRef", "mandateSignDate"
	 *  Ein Mandat ist die Ermächtigung des Zahlungsempfängers durch den Zahlungspflichtigen mit einer Lastschrift vereinbarte Beträge abzubuchen.
	 *  Die Mandatsreferenz wird durch den Zahlungsempfänger frei vergeben und muss eindeutig sein.
	 *  In der Regel wird sie durch die Debit-API erstellt, siehe Rückgabe bei "sessionCreate". 
	 *   
	 *  Wird eine eigene Mandatsreferenz übergeben, und existiert dieses Mandat noch nicht, kann das Datum der Mandatserteilung übergeben werden. Anderenfalls wird es ermittelt bzw.
	 *  das Datum des sessionApprove Aufrufs angenommen.   
	 * 
	 *  
	 *  Mehrfach-Mandate:
	 *  Parametername "mandateRecur", "payRecurState"
	 *  Durch "mandateRecur" wird mitgeteilt, ob eine einmalige Lastschrift "ONEOFF" oder eine von mehreren Lastschriften "RECURRING" angelegt wird. 
	 *  Mit "FINAL" wird ein ggf. bestehendes Mandat geschlossen, d.h. das aktuelle Mandat wird letzmalig benutzt.
	 *    
	 *  Wird ein neues einmaliges Sepamandat mit "ONEOFF" erstellt, enthält "payRecurState" immer den Status 'OOFF'. Die Mandatsreferenz darf danach nicht mehr benutzt werden.
	 *  Eine Mehrfachsession liefert "FRST" wenn das Mandat neu ist oder "RCUR" wenn es bereits benutzt wurde. Wurde mit "FINAL" das Mandat geschlossen, enthält
	 *  "payRecurState" den Wert "FNAL". Auch hier darf die Mandatsreferenz nicht mehr benutzt werden.
	 * 
	 *  Gläubiger-ID:
	 *  Parametername "creditorId"
	 *  Die Gläubiger-Identifikationsnummer ist eine eindeutige Identifizierung für Zahlungsempfänger von SEPA-Mandaten.
	 *  Sie ist für jeden Zahlungsempfänger verpflichtend und kann bei der Bundesbank kostenlos beantragt werden, siehe https://extranet.bundesbank.de/scp/ .
	 *  Die Gläubiger-ID ist im ControlCenter unter Kundendaten->Gläubiger-Identifikationsnummer hinterlegt.
	 *  Alternativ wird die Gläubiger-ID von micropayment bereitgestellt.
	 * 
	 *  Prenotification:
	 *  Der Endkunde muss vor der Belastung (siehe auch "payDate") in geeigneter Weise informiert werden. Ein geeigneter Textvorschlag kann wie folgt aussehen
	 *  (wobei die Platzhalter in {} durch die tatsächlichen Werte ersetzt werden müssen)
	 * 
	 *  "Unsere Forderung in Höhe von {$amount} ziehen wir mit der SEPA-Lastschrift zum Mandat Nr. {$mandateRef} zu der Gläubiger-Identifikationsnummer {$creditorId}
	 *   von Ihrem Konto IBAN {$iban} bei der {$bank} BIC {$bic} zum Fälligkeitstag {$payDate} ein. Wir bitten Sie für Kontodeckung zu sorgen."
	 *  
	 *  Vor jeder Folgelastschrift bei Mehrfach-Mandaten muss ebenfass der Kunde benachrichtigt werden.
	 * 
	 *  Neuerung sessionCredit:
	 *  Mit Hilfe dieser Funktion ist es möglich eine Auszahlung an Kunden ohne vorherige Lastschrift vorzunehmen. Dieses Feature muss gesondert freigeschaltet werden, und ist nur unter strengen Vorgaben verfügbar.
	 * 
	 *  Neuerungen zur EU-Geldtransferverordnung:
	 *  Ab dem 26.06.2017 tritt die EU Verordnung 2015/847 in Kraft. Hierfür ist es notwendig, für Zahlungspflichtige ausserhalb des europäischen Wirtschaftsraumes (EEA), die
	 *  Adressdaten der Zahlers zu ermitteln. Betroffen sind derzeit lediglich schweizer Kunden. Die Funktionalitäten hierfür wurden in den Methoden bankaccountSet, -Get, -Check sowie bankCheck ergänzt.
	 *  Für Stammkunden müssen diese Daten nachträglich übermittelt werden, andernfalls liefert sessionCreate einen der folgenden Fehler: 4112, 4113
	 * 
	 *  ------------------------------------------------------------------------------------
	 * 
	 *  Alle Funktionen die bisher mit Bankleitzahl/Kontonummer aufgerufen wurden, können jetzt alternativ mit IBAN/BIC bedient werden.
	 *  Bei Aufrufen ohne IBAN ist der Parametername "country" wichtig.
	 * 
	 *  Fehlercodes:
	 * 
	 *   4101	// Bankleitzahl ist ungültig
	 *   4102	// Kontonummer ist ungültig
	 *   4103	// Es wurde keine Kontoverbindung hinterlegt
	 *   4104	// Vorgang wurde bereits bestätigt
	 *   4105	// Kontoverbindung wurde gesperrt
	 *   4106	// Limit erreicht
	 *   4107	// kein offener Betrag oder übersteigt Sessionbetrag
	 *   4108	// Iban ist ungültig
	 *   4109	// BIC erforderlich oder ungültig
	 *   4110	// Der Zahldienst wird von der bezogenen Bank nicht unterstützt.
	 *   4111	// Banken aus diesem Land werden nicht unterstützt
	 *   4112	// Die Angabe des Kontoinhabers ist unzureichend, der volle Name ist notwendig
	 *   4113	// Iban gehört nicht zum europäischen Wirtschaftsraum, die Angabe der Kontoinhaber-Adresse ist notwendig
	 * 
	 *  ------------------------------------------------------------------------------------
	 *  
	 *  Hinweise zum TestModus: 
	 *  - Aktivierung der Testumgebung erfolgt in der jeweiligen Funktion über den Parameter `testMode=1`
	 *  - um automatisierte interne Prozesse und Ereignisse zu simulieren, stehen die ...Test-Funktionen zur Verfügung. Um eine vollumfängliche Lastschrift mit allen Eventualitäten zu simulieren sind folgende Funktionsaufrufe sinnvoll:
	 *    - customerCreate (Kunden anlegen)
	 *    - bankaccountSet (Bankverbindung anlegen)
	 *    - sessionCreate (Bezahlvorgang erzeugen)
	 *    - sessionApprove (Lastschrift bestätigen)
	 *    - sessionChargeTest (Abbuchung simulieren) 
	 *    - sessionReverseTest (Rücklastschrift simulieren) 
	 *    - sessionRechargeTest (Nachzahlung simulieren) 
	 *    - transactionCreate (Nachzahlung auf eig. Konto verbuchen) 
	 *    - sessionRefund (Rücküberweisung) 
	 *    - sessionRefundReverseTest (Retour der Rücküberweisung simulieren) 
	 *  - für den TestModus steht neben allen regulären, folgende Test-Bankverbindung zur Verfügung, um erfolgreiche Buchungen zu simulieren: 
	 *    - bankCode: 88888888
	 *    - accountNumber: z.B: 1234567890
	 *    - iban: z.B: DE89888888881234567890
	 *    - bic: TESTDE00XXX (optional)
	 *
	 * @copyright 2017 micropayment GmbH
	 * @link http://www.micropayment.de/
	 * @author Holger Heyne
	 * @version 2.3
	 * @created 2017-06-12 12:04:01
	 */
	interface IMcpDebitService_v2_3 {

		/**
		 * löscht alle Kunden und Transaktionen in der Testumgebung
		 *
		 * @param string $accessKey AccessKey aus dem Controlcenter
		 * @param integer $testMode Muss 1 sein
		 * 
		 * @return void 
		 */
		public function resetTest($accessKey, $testMode);

		/**
		 * legt neuen Kunden an
		 *
		 * @param string $accessKey AccessKey aus dem Controlcenter
		 * @param integer $testMode (default=0)  aktiviert Testumgebung
		 * @param string $customerId (default=null)  eigene eindeutige ID des Kunden, wird anderenfalls erzeugt
		 * @param array $freeParams (default=null)  Liste mit freien Parametern, die dem Kunden zugeordnet werden
		 * 
		 * @return array 
		 * @result string $customerId eigene oder erzeugte eindeutige ID des Kunden
		 */
		public function customerCreate($accessKey, $testMode=0, $customerId=null, $freeParams=null);

		/**
		 * ordnet weitere freie Parameter dem Kunden zu, oder ändert sie
		 *
		 * @param string $accessKey AccessKey aus dem Controlcenter
		 * @param integer $testMode (default=0)  aktiviert Testumgebung
		 * @param string $customerId eindeutige ID des Kunden
		 * @param array $freeParams (default=null)  Liste mit zusätzlichen freien Parametern
		 * 
		 * @return void 
		 */
		public function customerSet($accessKey, $testMode=0, $customerId, $freeParams=null);

		/**
		 * ermittelt alle freien Parameter des Kunden
		 *
		 * @param string $accessKey AccessKey aus dem Controlcenter
		 * @param integer $testMode (default=0)  aktiviert Testumgebung
		 * @param string $customerId ID des Kunden
		 * 
		 * @return array 
		 * @result array $freeParams (default=null)  Liste mit allen freien Parametern
		 */
		public function customerGet($accessKey, $testMode=0, $customerId);

		/**
		 * ermittelt alle Kunden
		 *
		 * @param string $accessKey AccessKey aus dem Controlcenter
		 * @param integer $testMode (default=0)  aktiviert Testumgebung
		 * @param integer $from (default=0)  Position des ersten auszugebenden Kunden
		 * @param integer $count (default=100)  Anzahl der auszugebenden Kunden
		 * 
		 * @return array 
		 * @result array $customerIdList Liste mit allen freien Parametern
		 * @result integer $count Anzahl der Kunden in der Liste
		 * @result integer $maxCount Gesamtanzahl aller Kunden
		 */
		public function customerList($accessKey, $testMode=0, $from=0, $count=100);

		/**
		 * erzeugt oder ändert Bankverbindung eines Kunden.
		 *  Die Adressdaten holder* sind optional, werden aber für IBAN aus Nicht-EWR-Ländern (derzeit nur Schweiz) benötigt.
		 *
		 * @param string $accessKey AccessKey aus dem Controlcenter
		 * @param integer $testMode (default=0)  aktiviert Testumgebung
		 * @param string $customerId ID des Kunden
		 * @param string $iban (default=null)  IBAN
		 * @param string $bic (default=null)  BIC, opt. bei deutscher IBAN
		 * @param string $country (default='DE')  Sitz der Bank, wenn IBAN leer ist
		 * @param string $bankCode (default=null)  Bankleitzahl, wenn IBAN leer ist
		 * @param string $accountNumber (default=null)  Kontonummer, wenn IBAN leer ist
		 * @param string $accountHolder Kontoinhaber
		 * @param string $holderAddress (default=null)  Kontoinhaber Adresse
		 * @param string $holderZip (default=null)  Kontoinhaber Postleitzahl
		 * @param string $holderCity (default=null)  Kontoinhaber Ort
		 * @param string $holderCountry (default=null)  Kontoinhaber Land
		 * 
		 * @return array 
		 * @result string $iban IBAN
		 * @result string $bic BIC
		 * @result string $country Sitz der Bank
		 * @result string $bankName Name der Bank
		 * @result string $bankCode Bankleitzahl
		 * @result string $accountNumber Kontonummer
		 * @result string $accountHolder Kontoinhaber
		 * @result boolean $eeaStatus True, wenn Iban zum EWR gehört, andernfalls ist die Angabe der Kontoinhaber-Adresse notwendig
		 * @result string $barStatus Sperr-Status der Kontoverbindung
		 */
		public function bankaccountSet($accessKey, $testMode=0, $customerId, $iban=null, $bic=null, $country='DE', $bankCode=null, $accountNumber=null, $accountHolder, $holderAddress=null, $holderZip=null, $holderCity=null, $holderCountry=null);

		/**
		 * ermittelt die Bankverbindung des Kunden
		 *
		 * @param string $accessKey AccessKey aus dem Controlcenter
		 * @param integer $testMode (default=0)  aktiviert Testumgebung
		 * @param string $customerId ID des Kunden
		 * 
		 * @return array 
		 * @result string $iban IBAN
		 * @result string $bic BIC
		 * @result string $country Sitz der Bank
		 * @result string $bankName Name der Bank
		 * @result string $bankCode Bankleitzahl
		 * @result string $accountNumber Kontonummer
		 * @result string $accountHolder Kontoinhaber
		 * @result string $holderAddress (default=null)  Kontoinhaber Adresse, Strasse und Hausnummer
		 * @result string $holderZip (default=null)  Kontoinhaber Postleitzahl
		 * @result string $holderCity (default=null)  Kontoinhaber Ort
		 * @result string $holderCountry (default=null)  Kontoinhaber Land
		 * @result boolean $eeaStatus True, wenn Iban zum EWR gehört, andernfalls ist die Angabe der Kontoinhaber-Adresse notwendig
		 * @result string $barStatus Sperr-Status der Kontoverbindung
		 */
		public function bankaccountGet($accessKey, $testMode=0, $customerId);

		/**
		 * prüft Bankleitzahl und ermittelt Banknamen
		 *
		 * @param string $accessKey AccessKey aus dem Controlcenter
		 * @param integer $testMode (default=0)  aktiviert Testumgebung
		 * @param string $iban (default=null)  IBAN
		 * @param string $bic (default=null)  BIC, alternativ zur IBAN
		 * @param string $country (default='DE')  Sitz der Bank, alternativ zu IBAN und BIC
		 * @param string $bankCode (default=null)  Bankleitzahl, alternativ zu IBAN und BIC
		 * 
		 * @return array 
		 * @result string $bic BIC
		 * @result string $country Sitz der Bank
		 * @result string $bankName Name der Bank
		 * @result string $bankCode Bankleitzahl
		 * @result boolean $eeaStatus True, wenn Bank zum EWR gehört, andernfalls ist die Angabe der Kontoinhaber-Adresse notwendig
		 * @result string $orderStatus Autragsart der Bank ALL = Lastschriften, Überweisungen, CREDIT = nur Überweisung, DENIED = Gesperrt für Zahlungverkehr,
		 */
		public function bankCheck($accessKey, $testMode=0, $iban=null, $bic=null, $country='DE', $bankCode=null);

		/**
		 * prüft Bankverbindung und ermittelt Banknamen
		 *
		 * @param string $accessKey AccessKey aus dem Controlcenter
		 * @param integer $testMode (default=0)  aktiviert Testumgebung
		 * @param string $iban (default=null)  IBAN
		 * @param string $bic (default=null)  BIC, opt. bei deutscher IBAN
		 * @param string $country (default='DE')  Sitz der Bank, wenn IBAN leer ist
		 * @param string $bankCode (default=null)  Bankleitzahl, wenn IBAN leer ist
		 * @param string $accountNumber (default=null)  Kontonummer, wenn IBAN leer ist
		 * 
		 * @return array 
		 * @result string $iban IBAN
		 * @result string $bic BIC
		 * @result string $country Sitz der Bank
		 * @result string $bankName Name der Bank
		 * @result string $bankCode Bankleitzahl
		 * @result string $orderStatus Autragsart der Bank ALL = Lastschriften, Überweisungen, CREDIT = nur Überweisung, DENIED = Gesperrt für Zahlungverkehr,
		 * @result string $accountNumber Kontonummer
		 * @result boolean $eeaStatus True, wenn Iban zum EWR gehört, andernfalls ist die Angabe der Kontoinhaber-Adresse notwendig
		 * @result string $barStatus Sperr-Status der Kontoverbindung
		 */
		public function bankaccountCheck($accessKey, $testMode=0, $iban=null, $bic=null, $country='DE', $bankCode=null, $accountNumber=null);

		/**
		 * Sperrt Bankverbindung oder gibt sie frei
		 *
		 * @param string $accessKey AccessKey aus dem Controlcenter
		 * @param integer $testMode (default=0)  aktiviert Testumgebung
		 * @param string $iban (default=null)  IBAN
		 * @param string $bic (default=null)  BIC, opt. bei deutscher IBAN
		 * @param string $country (default='DE')  Sitz der Bank, wenn IBAN leer ist
		 * @param string $bankCode (default=null)  Bankleitzahl, wenn IBAN leer ist
		 * @param string $accountNumber (default=null)  Kontonummer, wenn IBAN leer ist
		 * @param string $barStatus Sperr-Status BARRED, ALLOWED
		 * 
		 * @return void 
		 */
		public function bankaccountBar($accessKey, $testMode=0, $iban=null, $bic=null, $country='DE', $bankCode=null, $accountNumber=null, $barStatus);

		/**
		 * erzeugt oder ändert Adressdaten eines Kunden
		 *
		 * @param string $accessKey AccessKey aus dem Controlcenter
		 * @param integer $testMode (default=0)  aktiviert Testumgebung
		 * @param string $customerId ID des Kunden
		 * @param string $firstName (default=null)  Vorname
		 * @param string $surName (default=null)  Nachname
		 * @param string $company (default=null)  Firma
		 * @param string $street (default=null)  Strasse und Hausnummer
		 * @param string $zip (default=null)  Postleitzahl
		 * @param string $city (default=null)  Ort
		 * @param string $country (default='DE')  Land
		 * 
		 * @return void 
		 */
		public function addressSet($accessKey, $testMode=0, $customerId, $firstName=null, $surName=null, $company=null, $street=null, $zip=null, $city=null, $country='DE');

		/**
		 * ermittelt die Adresse des Kunden
		 *
		 * @param string $accessKey AccessKey aus dem Controlcenter
		 * @param integer $testMode (default=0)  aktiviert Testumgebung
		 * @param string $customerId ID des Kunden
		 * 
		 * @return array 
		 * @result string $firstName Vorname
		 * @result string $surName Nachname
		 * @result string $company Firma
		 * @result string $street Strasse und Hausnummer
		 * @result string $zip Postleitzahl
		 * @result string $city Ort
		 * @result string $country Land
		 */
		public function addressGet($accessKey, $testMode=0, $customerId);

		/**
		 * erzeugt oder ändert Kontaktdaten eines Kunden
		 *
		 * @param string $accessKey AccessKey aus dem Controlcenter
		 * @param integer $testMode (default=0)  aktiviert Testumgebung
		 * @param string $customerId ID des Kunden
		 * @param string $email (default=null)  Emailadresse des Kunden
		 * @param string $phone (default=null)  Festnetzanschluss
		 * @param string $mobile (default=null)  Handynummer
		 * 
		 * @return void 
		 */
		public function contactDataSet($accessKey, $testMode=0, $customerId, $email=null, $phone=null, $mobile=null);

		/**
		 * ermittelt die Kontaktdaten des Kunden
		 *
		 * @param string $accessKey AccessKey aus dem Controlcenter
		 * @param integer $testMode (default=0)  aktiviert Testumgebung
		 * @param string $customerId ID des Kunden
		 * 
		 * @return array 
		 * @result string $email Emailadresse
		 * @result string $phone Festnetzanschluss
		 * @result string $mobile Handynummer
		 */
		public function contactDataGet($accessKey, $testMode=0, $customerId);

		/**
		 * erzeugt einen neuen Bezahlvorgang
		 *  löst die Benachrichtigung sessionStatus mit dem Status "INIT" bzw. "REINIT" aus
		 *
		 * @param string $accessKey AccessKey aus dem Controlcenter
		 * @param integer $testMode (default=0)  aktiviert Testumgebung
		 * @param string $customerId ID des Kunden
		 * @param string $sessionId (default='')  eigene eindeutige ID des Vorgangs, wird anderenfalls erzeugt
		 * @param string $project das Projektkürzel für den Vorgang
		 * @param string $projectCampaign (default='')  ein Kampagnenkürzel des Projektbetreibers
		 * @param string $account (default='')  Account des beteiligten Webmasters
		 * @param string $webmasterCampaign (default='')  ein Kampagnenkürzel des Webmasters
		 * @param integer $amount (default=0)  abzurechnender Betrag in Cent der jeweiligen Währung
		 * @param string $currency (default='EUR')  Währung
		 * @param string $title (default='')  Bezeichnung der zu kaufenden Sache
		 * @param string $payText (default='')  Abbuchungstext der Lastschrift
		 * @param string $ip (default='')  IP des Benutzers
		 * @param array $freeParams (default=null)  Liste mit freien Parametern, die dem Vorgang zugeordnet werden
		 * @param string $mandateRef (default=null)  SEPA Mandats Referenz, wird ansonsten erzeugt
		 * @param string $mandateSignDate (default=null)  Datum der Mandatserteilung, ansonsten sessionApprove-Datum
		 * @param string $mandateRecur (default=null)  SEPA Mandat für einzelne - Standard für neue Mandate: "ONEOFF", oder wiederkehrende Zahlungen: "RECURRING" oder "FINAL"
		 * 
		 * @return array 
		 * @result string $sessionId eigene oder erzeugte eindeutige ID des Vorgangs
		 * @result string $status Vorgangsstatus "INIT" oder "REINIT"
		 * @result string $expire Ablaufzeit der Bestätigung
		 * @result integer $amount übergebener Betrag bzw. Standard aus Konfiguration in Cent
		 * @result string $currency übergebene Währung bzw. "EUR"
		 * @result string $title übergebene Kaufsache bzw. Standard aus Konfiguration
		 * @result string $payToken Zuordnungsschlüssel im Lastschrifttext
		 * @result string $payText Abbuchungstext der Lastschrift
		 * @result string $payAmount Zahlbetrag in EUR-Cent
		 * @result string $creditorId SEPA Gläubiger ID
		 * @result string $mandateRef SEPA Mandats Referenz
		 * @result string $mandateSignDate SEPA Datum der Mandatserteilung oder <leer>
		 * @result string $payRecurState SEPA Wiederholungs-Status der Zahlung: "OOFF", "FRST", "RCUR", "FNAL"
		 * @result string $payDate SEPA Datum der Belastung
		 */
		public function sessionCreate($accessKey, $testMode=0, $customerId, $sessionId='', $project, $projectCampaign='', $account='', $webmasterCampaign='', $amount=0, $currency='EUR', $title='', $payText='', $ip='', $freeParams=null, $mandateRef=null, $mandateSignDate=null, $mandateRecur=null);

		/**
		 * ordnet weitere freie Parameter der Session zu, oder ändert sie
		 *
		 * @param string $accessKey AccessKey aus dem Controlcenter
		 * @param integer $testMode (default=0)  aktiviert Testumgebung
		 * @param string $sessionId ID des Vorgangs
		 * @param array $freeParams (default=null)  Liste mit zusätzlichen freien Parametern
		 * 
		 * @return void 
		 */
		public function sessionSet($accessKey, $testMode=0, $sessionId, $freeParams=null);

		/**
		 * ermittelt Daten eines Bezahlvorgangs
		 *
		 * @param string $accessKey AccessKey aus dem Controlcenter
		 * @param integer $testMode (default=0)  aktiviert Testumgebung
		 * @param string $sessionId ID des Vorgangs
		 * 
		 * @return array 
		 * @result string $status Vorgangsstatus "INIT", "REINIT", "APPROVED", "CHARGED", "REVERSED" oder "RECHARGED"
		 * @result string $expire Ablaufzeit bzw. Bestätigung des Vorgangs
		 * @result string $statusDetail Beschreibung für gescheiterte Transaktionen
		 * @result string $customerId ID des Kunden
		 * @result string $project zugeordnetes Projekt
		 * @result string $projectCampaign zugeordnete Projektkampagne
		 * @result string $account zugeordneter Webmasteraccount
		 * @result string $webmasterCampaign zugeordnete Webmasterkampagne
		 * @result integer $amount übergebener Betrag bzw. Standard aus Konfiguration in Cent
		 * @result string $currency übergebene Währung bzw. "EUR"
		 * @result string $title übergebene Kaufsache bzw. Standard aus Konfiguration
		 * @result string $payToken Zuordnungsschlüssel im Lastschrifttext
		 * @result string $payText Abbuchungstext der Lastschrift
		 * @result integer $payAmount Zahlbetrag in EUR-Cent
		 * @result integer $openAmount offener, noch zu zahlender Betrag der Session in EUR-Cent
		 * @result integer $creditAmount bereits ausgezahlter Betrag der Session in EUR-Cent
		 * @result string $ip übergebene IP des Benutzers
		 * @result array $freeParams (default=null)  Liste mit allen freien Parametern
		 * @result string $creditorId SEPA Gläubiger ID
		 * @result string $mandateRef SEPA Mandats Referenz
		 * @result string $mandateSignDate SEPA Datum der Mandatserteilung oder <leer>
		 * @result string $payRecurState SEPA Wiederholungs-Status der Zahlung: "OOFF", "FRST", "RCUR", "FNAL"
		 * @result string $payDate SEPA Datum der Belastung
		 */
		public function sessionGet($accessKey, $testMode=0, $sessionId);

		/**
		 * bestätigt den Lastschrifteinzug eines Vorgangs
		 *  löst die Benachrichtigung sessionStatus mit dem Status "APPROVED" aus
		 *
		 * @param string $accessKey AccessKey aus dem Controlcenter
		 * @param integer $testMode (default=0)  aktiviert Testumgebung
		 * @param string $sessionId ID des Vorgangs
		 * 
		 * @return array 
		 * @result string $status Vorgangsstatus "APPROVED" oder "FAILED"
		 * @result string $expire Zeitpunkt der Bestätigung
		 */
		public function sessionApprove($accessKey, $testMode=0, $sessionId);

		/**
		 * Erzeugt eine Gutschrift an den Kunden, unabhängig von Status und Zahlstand der Session
		 *
		 * @param string $accessKey AccessKey aus dem Controlcenter
		 * @param integer $testMode (default=0)  aktiviert Testumgebung
		 * @param string $sessionId ID des Vorgangs
		 * @param integer $amount Überweisungsbetrag in EUR-Cent
		 * @param string $payText (default=null)  Buchungstext, stdm. wird der ursprüngliche Buchungstext verwendet
		 * 
		 * @return array 
		 * @result integer $amount Überweisungsbetrag in EUR-Cent
		 * @result string $payText Buchungstext
		 */
		public function sessionCredit($accessKey, $testMode=0, $sessionId, $amount, $payText=null);

		/**
		 * ermittelt alle Bezahlvorgänge eines Kunden
		 *
		 * @param string $accessKey AccessKey aus dem Controlcenter
		 * @param integer $testMode (default=0)  aktiviert Testumgebung
		 * @param string $customerId (default=null)  ID des Kunden
		 * 
		 * @return array 
		 * @result integer $count Anzahl der Einträge in sessionIdList
		 * @result array $sessionIdList (default=null)  0-indizierte Liste mit Vorgang-IDs
		 */
		public function sessionList($accessKey, $testMode=0, $customerId=null);

		/**
		 * simuliert die Abbuchung für alle bestätigten Vorgänge
		 *  erzeugt für jede bestätigte Session eine neue Transaktion mit dem Typ "BOOKING" und löst die Benachrichtigung transactionCreate aus
		 *  löst die Benachrichtigung sessionStatus mit dem Status "CHARGED" aus
		 *
		 * @param string $accessKey AccessKey aus dem Controlcenter
		 * @param integer $testMode (default=0)  muss 1 sein
		 * 
		 * @return array 
		 * @result integer $count Anzahl der gebuchten Vorgänge
		 */
		public function sessionChargeTest($accessKey, $testMode=0);

		/**
		 * simuliert Stornierung eines einzelnen Vorgangs
		 *  erzeugt eine neue Transaktion mit dem Typ "REVERSAL" und löst die Benachrichtigung transactionCreate aus
		 *  löst die Benachrichtigung sessionStatus mit dem Status "REVERSED" aus
		 *
		 * @param string $accessKey AccessKey aus dem Controlcenter
		 * @param integer $testMode (default=0)  muss 1 sein
		 * @param string $sessionId ID des Vorgangs
		 * @param integer $reverseCode (default=0)  Textschlüssel der Bank z.B. 9055 - Widerspruch
		 * @param string $reverseReason (default='UNKNOWN')  Rücklastschriftgrund "UNSPECIFIED", "ACCOUNT_EXPIRED", "ACCOUNT_WRONG", "NOT_AUTHORIZED", "CONTRADICTED", "UNKNOWN"
		 * 
		 * @return array 
		 * @result integer $amount stornierter Betrag inkl. Gebühr in EUR-Cent
		 */
		public function sessionReverseTest($accessKey, $testMode=0, $sessionId, $reverseCode=0, $reverseReason='UNKNOWN');

		/**
		 * simuliert die komplette Nachzahlung eines stornierten Vorgangs
		 *  erzeugt eine neue Transaktion mit dem Typ "BACKPAY" und löst die Benachrichtigung transactionCreate aus
		 *  löst die Benachrichtigung sessionStatus mit dem Status "RECHARGED" aus, wenn der gesamte offene Betrag beglichen wurde
		 *
		 * @param string $accessKey AccessKey aus dem Controlcenter
		 * @param integer $testMode (default=0)  muss 1 sein
		 * @param string $sessionId ID des Vorgangs
		 * @param integer $amount (default=null)  optional der nachgezahlte Teilbetrag in EUR-Cent
		 * 
		 * @return array 
		 * @result integer $amount gebuchter Betrag in EUR-Cent
		 */
		public function sessionRechargeTest($accessKey, $testMode=0, $sessionId, $amount=null);

		/**
		 * Veranlasst eine (Teil-)Gutschrift und überweist sie zurück
		 *  erzeugt eine neue Transaktion mit dem Typ "REFUND" und löst die Benachrichtigung transactionCreate aus
		 *
		 * @param string $accessKey AccessKey aus dem Controlcenter
		 * @param integer $testMode (default=0)  aktiviert Testumgebung
		 * @param string $sessionId ID der zugehörigen Session
		 * @param string $bankCode (default=null)  Bankleitzahl, alternativ zur IBAN
		 * @param string $accountNumber (default=null)  Kontonummer, alternativ zur IBAN
		 * @param string $accountHolder (default=null)  Kontoinhaber
		 * @param integer $amount (default=null)  Überweisungsbetrag in EUR-Cent, stdm. wird der gesamte eingegangene Betrag überwiesen
		 * @param string $payText (default=null)  Buchungstext, stdm. wird der ursprüngliche Buchungstext verwendet
		 * @param string $iban (default=null)  IBAN des alternativen Empfängers
		 * @param string $bic (default=null)  BIC, opt. bei deutscher IBAN
		 * @param string $country (default='DE')  Sitz der Bank, alternativ zur IBAN
		 * 
		 * @return array 
		 * @result integer $amount Überweisungsbetrag in EUR-Cent
		 * @result string $payText Buchungstext
		 */
		public function sessionRefund($accessKey, $testMode=0, $sessionId, $bankCode=null, $accountNumber=null, $accountHolder=null, $amount=null, $payText=null, $iban=null, $bic=null, $country='DE');

		/**
		 * simuliert Stornierung der letzten Gutschrift,
		 *  erzeugt eine neue Transaktion mit dem Typ "REFUNDREVERSAL" und löst die Benachrichtigung transactionCreate aus
		 *
		 * @param string $accessKey AccessKey aus dem Controlcenter
		 * @param integer $testMode (default=0)  muss 1 sein
		 * @param string $sessionId ID des Vorgangs
		 * 
		 * @return array 
		 * @result integer $amount stornierter Gutschriftbetrag in EUR-Cent
		 */
		public function sessionRefundReverseTest($accessKey, $testMode=0, $sessionId);

		/**
		 * erstellt eine Transaktion vom Typ "EXTERNAL"
		 *  löst die Benachrichtigung transactionCreate aus
		 *
		 * @param string $accessKey AccessKey aus dem Controlcenter
		 * @param integer $testMode (default=0)  aktiviert Testumgebung
		 * @param string $sessionId ID der zugehörigen Session
		 * @param string $date (default=null)  Datum der Transaktion
		 * @param integer $amount Transaktionsbetrag in EUR-Cent
		 * @param string $description (default='""')  Beschreibungstext
		 * 
		 * @return void 
		 */
		public function transactionCreate($accessKey, $testMode=0, $sessionId, $date=null, $amount, $description='""');

		/**
		 * ermittelt alle Transaktionen für einen Bezahlvorgang
		 *
		 * @param string $accessKey AccessKey aus dem Controlcenter
		 * @param integer $testMode (default=0)  aktiviert Testumgebung
		 * @param string $sessionId ID des Vorgangs
		 * 
		 * @return array 
		 * @result integer $count Anzahl der Einträge in transactionIdList
		 * @result array $transactionIdList 0-indizierte Liste mit Transaktions-IDs
		 */
		public function transactionList($accessKey, $testMode=0, $sessionId);

		/**
		 * ermittelt Daten einer Transaktion
		 *
		 * @param string $accessKey AccessKey aus dem Controlcenter
		 * @param integer $testMode (default=0)  aktiviert Testumgebung
		 * @param string $transactionId ID des Vorgangs
		 * 
		 * @return array 
		 * @result string $sessionId ID des Vorgangs
		 * @result string $date Datum der Transaktion
		 * @result string $type Art der Transaktion "BOOKING", "REVERSAL", "BACKPAY", "EXTERNAL", "REFUND", "REFUNDREVERSAL", "CREDIT"
		 * @result string $reverseReason (default=null)  Bei $type = "REVERSAL" Art der Rücklastschrift "UNSPECIFIED", "ACCOUNT_EXPIRED", "ACCOUNT_WRONG", "NOT_AUTHORIZED", "CONTRADICTED", "UNKNOWN"
		 * @result integer $amount Transaktionsbetrag in EUR-Cent
		 * @result string $description Beschreibungstext
		 */
		public function transactionGet($accessKey, $testMode=0, $transactionId);

	}

?>